//
//  I2C制御 4桁7セグメントLEDコントローラ (I2Cスレーブ)  I2C化キット
//    copyright (c) 2013 T.Nakao      http://www.wsnak.com
//
//    12/03/19
//    12/05/12 1桁表示コマンド追加
//    13/05/30 関数名 誤記訂正
//

/*
  7セグメントLED4桁をダイナミックドライブ接続したものを
  I2Cデバイス化する、Arduino使用のコントローラ
  汎用、個別のLEDをDD接続した物、OSL40562-LRなど使用可能
*/

#include <Wire.h>     // I2C
#include <wDisplay.h> // 7セグメントLEDドライバ

wD7S4Led L7Seg;       // 7セグメントLEDオブジェクト

byte I2CAdrs = 0x10;  // I2Cスレーブアドレス


// 初期化
void setup(void) {

  L7Seg.init();               // ドライバ初期化

  Wire.begin(I2CAdrs);        // I2Cスレーブ初期化
  Wire.onReceive(hI2CSlvRcv); // I2Cスレーブ受信ハンドラ登録

  pinMode(13, OUTPUT);        // オンボードLED

//  L7Seg.DigRvs = 0;
  L7Seg.setZeroSup(true);     // ゼロサプレス
  L7Seg.setNum(120);
}


// メインループ
void loop(void) {
  L7Seg.process();            // 7セグlED ダイナミックドライブ
}


// スレーブ受信時のハンドラ
void hI2CSlvRcv(int cnt) {
  while(cnt >= 2) {
    cmdProc();
    cnt -= 2;
  }

  if(cnt == 1) {    // 端数(イリーガル・フォーマット)
    Wire.read();    // 読み捨て
  }
}


void cmdProc() {
  int num;
  byte cmd, para;
  byte buf[2];


  buf[0] = Wire.read();               // データ読み出し
  buf[1] = Wire.read();               //

  if(buf[0] & 0x80) {
    // 制御コマンド
    cmd = buf[0];
    para = buf[1];

    switch(cmd) {
      case 0x80:    // 少数点表示コマンド
        if(para & 1) {
          // 1の桁 少数点表示
          L7Seg.setDP(1, 1);
        } else {
          // 1の桁 少数点なし
          L7Seg.setDP(1, 0);
        }
        if(para & 2) {
          // 10の桁 少数点表示
          L7Seg.setDP(2, 1);
        } else {
          // 10の桁 少数点なし
          L7Seg.setDP(2, 0);
        }
        if(para & 4) {
          // 100の桁 少数点表示
          L7Seg.setDP(3, 1);
        } else {
          // 100の桁 少数点なし
          L7Seg.setDP(3, 0);
        }
        if(para & 8) {
          // 1000の桁 少数点表示
          L7Seg.setDP(4, 1);
        } else {
          // 1000の桁 少数点なし
          L7Seg.setDP(4, 0);
        }
        break;

      case 0x81:    // ゼロパディングコマンド
        if(para & 1) {
          // ゼロパディングあり
          L7Seg.setZeroSup(0);    // ゼロサプレスなし
        } else {
          // ゼロパディングなし
          L7Seg.setZeroSup(1);    // ゼロサプレスあり
        }
        break;

      case 0x82:    // 表示コマンド
        if(para & 1) {
          // 通常点灯
          L7Seg.disp(1);          // 点灯
          digitalWrite(13, HIGH); // LED ON
        } else {
          // 全表示消灯
          L7Seg.disp(0);          // 消灯
          digitalWrite(13, LOW);  // LED OFF
        }
        break;

      case 0x88:  // 1桁表示コマンド(1の桁)
      case 0x89:  // 1桁表示コマンド(10の桁)
      case 0x8A:  // 1桁表示コマンド(100の桁)
      case 0x8B:  // 1桁表示コマンド(1000の桁)
        int ix;
        ix = cmd & 0x03;    // 桁インデックス

        if(para > 17) {           // 18～
          L7Seg.digit[ix] = 17;   // ブランク
        } else {
          L7Seg.digit[ix] = para;
        }
        L7Seg.setSegPat();        // セグメントパターン設定
        break;
    }
  } else {
    // 数値設定コマンド
    num = ((int)buf[0] << 8) + buf[1];  // 表示する数値
    L7Seg.setNum(num);      // 数値設定、表示更新
  }
}
